/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.ste.rdstmc.junit.tools;

import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.MutableSet;
import de.bsvrz.dav.daf.main.config.MutableSetChangeListener;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.ste.rdstmc.meldungserstellung.VerwaltungMeldungen;

public class WarteAufNeueDaten implements MutableSetChangeListener, ClientReceiverInterface 
{
  private ClientDavInterface _connection;
  private DataDescription _dd;
  
  private Collection<String> _erwartetePids = new HashSet<String>();
  
  private List<SystemObject> _angemeldeteObjekte = new LinkedList<SystemObject>();
  
  private Map<String, ResultData> _neueDaten = new HashMap<String, ResultData>();
  
  public Map<String, ResultData> getNeueDaten()
  {
    return Collections.unmodifiableMap(_neueDaten);
  }
  
  /**
   * Es werden die neu in einer Menge hinzukommenden Objekte bestimmt. Auf neue Objekte
   * wird gewartet, bis die erwarteten Objekte (Angabe ber PID) in der Menge sind ode
   * bis ein Timeout verstrichen ist.
   * Zu den neuen Objekten werden die aktuellen Daten gemss der bergenenen 
   * Datenbeschreibung bestimmt
   * 
   * @param connection die DAV-Verbindung
   * @param menge die Menge, deren Objekte berwacht wird 
   * @param erwartetePids die PIDs der Objekte, von denen erwartet wird, dass sie neu in die Menge 
   * hinzukommen
   * @param dd Datenbeschreibung, fr die die aktuellen Datenstze der neuen Objekte bestimmt werden
   * @param timeout Timeout - solange wird maximal auf neue Objekte gewartet - wird hier 0 bergeben, 
   * wird unendlich lange gewartet (dead lock Gefahr)
   */
  public WarteAufNeueDaten(ClientDavInterface connection, MutableSet menge, 
      String[] erwartetePids, DataDescription dd, long timeout) 
  {
    _connection = connection;
    _dd = dd;
    
    for (String pid: erwartetePids)
    {
      if (!pid.startsWith(VerwaltungMeldungen.PID_PREFIX))
      {
        throw new IllegalArgumentException("Die bergebenen PIDs mssen mit " + 
            VerwaltungMeldungen.PID_PREFIX + " beginnen");
      }
      _erwartetePids.add(pid);
    }
    
    menge.addChangeListener(this);
    
    try
    {
      synchronized (this)
      {
          wait(timeout);
      }
    }
    catch(InterruptedException e)
    {
      throw new RuntimeException("Unerwarteter Abbruch", e);
    }
    finally
    {
      menge.removeChangeListener(this);
      _connection.unsubscribeReceiver(this, _angemeldeteObjekte, _dd);
    }
  }
  
  /*
   * (Kein Javadoc)
   * @see de.bsvrz.dav.daf.main.config.MutableSetChangeListener#update(de.bsvrz.dav.daf.main.config.MutableSet, de.bsvrz.dav.daf.main.config.SystemObject[], de.bsvrz.dav.daf.main.config.SystemObject[])
   */
  public void update(MutableSet set, SystemObject[] addedObjects, 
      SystemObject[] removedObjects)
  {
    for (SystemObject objekt: removedObjects)
    {
      if (_angemeldeteObjekte.remove(objekt))
      {
        _connection.unsubscribeReceiver(this, objekt, _dd);
      }
      _neueDaten.remove(objekt);
      _erwartetePids.remove(objekt.getPid());
    }
    for (SystemObject objekt: addedObjects)
    {
      _connection.subscribeReceiver(this, objekt, _dd, ReceiveOptions.normal(), ReceiverRole.receiver());
      _angemeldeteObjekte.add(objekt);
    }
  }
  
  /*
   * (Kein Javadoc)
   * @see de.bsvrz.dav.daf.main.ClientReceiverInterface#update(de.bsvrz.dav.daf.main.ResultData[])
   */
  public void update(ResultData[] resultData)
  {
    for (ResultData data: resultData)
    {
      if (data.hasData())
      {
        SystemObject objekt = data.getObject();
        if (_angemeldeteObjekte.contains(objekt))
        {
          String pid = objekt.getPid();
          _neueDaten.put(pid, data);
          if (_erwartetePids.contains(pid))
          {
            _erwartetePids.remove(pid);
            if (_erwartetePids.isEmpty())
            {
              synchronized(this)
              {
                notify();
              }
            }
          }
        }
      }
    }
  }
  
}
