/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
 
package de.bsvrz.ste.rdstmc.situation;

import java.util.List;

import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

import de.bsvrz.ste.rdstmc.meldungserstellung.AbstraktVewDynObj;

/**
 * Abtrakte Klasse zur Verwaltung von dynamischen DAV-Objekten in Netzen.
 * Diese abstrakte Klasse stellt Funktionalitt zur Vefgung um generische Instanzen 
 * fr dynamische DAV-Objekte, die in dynamischen Mengen von allen Unternetzen eines
 * bestimmeten Typs eines Netzes enthalten sind, zu verwalten<br>.
 * Die verwalteten Instanzen werden in einer (nich modifizierbaren) Map mit der PID
 * des dynamischen Objekts als Schlssel zur Verfgung gestellt. Diese Map wird 
 * entsprechend den nderungen in der dynamischen Objektmengen und den Lschungen 
 * (Invalidierungen) der dynamischen Objekte aktualisiert.<br>
 * Zur Implementierung einer effektiven Verwaltung gengt die Implementierung
 * der Methode <code>erzeugeObjekt</code> und aufrufen der Methode <code>initialisiereNetz</code>
 * z. B. im Konstruktor der beerbenden Klasse.
 * 
 * @author Dambach Werke GmbH
 * @version $Revision: 1.1 $ / $Date: 2008/10/31 13:49:24 $ / ($Author: Sans $)
 */
public abstract class AbstraktNetzVewDynObj<T> extends AbstraktVewDynObj<T>
{
  /**
   * Debug-Logger fr Logging-Ausgaben
   */
  private final static Debug _debug = Debug.getLogger();
        
   /**
   * Initialisierte aus dem bergebene Netz - rekursiv - alle enthaltenen Objekte der angegebenen dynamische Mengen,
   * die durch den bergebenen Mengentyp bestimmt werden
   *  
   * @param netz das Wurzel-Objekt vom Typ typ.netz
   * @param pidMengenTyp der Unterryp des Typs typ.netz, dessen Objekte Besitzer der die dynamische Menge sind 
   * @param mengenBezeichner Bezeichner (Name) der Menge
   * @return true, wenn mindestens ein Objekt vom Typ typ.verkehrsModellNetz initialisiert 
   * wurde, false sonst
   */
  protected boolean initialisiereNetz(ConfigurationObject netz, String pidMengenTyp, String mengenBezeichner)
  {
    boolean result = false;
    
    if (!netz.isOfType("typ.netz"))
    {
      throw new IllegalArgumentException("Objekt " + netz + " ist nicht vom Typ typ.netz");
    }

    if (netz.isOfType(pidMengenTyp))
    {
      initialisiere(netz, mengenBezeichner);
      result = true;
    }
    
    List<SystemObject> nbt = netz.getNonMutableSet("NetzBestandTeile").getElements();
    if (nbt == null)
    {
      throw new RuntimeException("Menge \"NetzBestandTeile\" nicht vorhanden in " + netz);      
    }
    
    for (SystemObject obj: nbt)
    {
      if (obj.isOfType("typ.netz"))
      {
        result |= initialisiereNetz((ConfigurationObject)obj, pidMengenTyp, mengenBezeichner);
      }
    }
    return result;
  }
   
}
